#define __NEW_STARLET 1

#pragma module PCA_DUMP "X01-00"

/*
**++
**
** Copyright (c) 2014 James F. Duff.  ALL RIGHTS RESERVED.
**
** License Terms and Conditions:
**
** You may freely copy and distribute verbatim copies of this software package
** as you receive it, in any medium, provided that you conspicuously and
** appropriately publish on each copy this copyright notice and
** disclaimer of warranty; keep intact all the notices that refer to these
** license terms and conditions and to the absence of any warranty; and give
** any other recipients of the software package a copy of these license
** terms and conditions along with the software.
**
** You may modify the software for your own use, but you MAY NOT distribute
** modified copies of the software.
**
** By copying or distributing the software package you indicate your
** acceptance of this license to do so, and all its terms and conditions.
**
** Each time you redistribute the software package the recipient
** automatically receives a license from the original licensor to copy,
** distribute or modify the software package subject to these terms and
** conditions. You may not impose any further restrictions on the recipients'
** exercise of the rights granted herein.
**
** BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
** FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
** OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDER PROVIDE THE SOFTWARE
** "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
** INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
** AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY
** AND PERFORMANCE OF THE SOFTWARE IS WITH YOU.  SHOULD THE SOFTWARE PROVE
** DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
** CORRECTION.
**
** YOU WARRANT THAT IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO
** YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR
** CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE
** SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE
** SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE, FIRMWARE OR HARDWARE), EVEN IF
** SUCH HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
**
** NO TITLE TO AND OWNERSHIP OF THE SOFTWARE PACKAGE IS HEREBY TRANSFERRED.
**
** Send suggestions, bug reports, money ;-) to jim@eight-cubed.com
**
** Now that's out of the way, on with the code...
**
**--
*/

/*
**++
**
**  MODULE DESCRIPTION:
**
**	This program reads the file produced by PCA.C and writes out a sorted
**	by count list of PCs.  See PCA.C for further information.
**
**
**  AUTHORS:
**
**      James F. Duff
**
**  CREATION DATE:
**
**      10-Jan-2014
**
**  MODIFICATION HISTORY:
**
**      X01-00      Jim Duff        17-Mar-2014
**      Original version of module.
**
**--
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ssdef.h>
#include <stsdef.h>
#include <efndef.h>
#include <secdef.h>
#include <va_rangedef.h>
#include <descrip.h>
#include <rms.h>
#include <lib$routines.h>
#include <starlet.h>

#include "pca.h"

#define errchk_stop(arg) if (!$VMS_STATUS_SUCCESS(arg)) (void)lib$stop(arg);


/******************************************************************************/
static int compare (const void *a, const void *b) {
/*
** Comparison routine for qsort.  Ensure we do descending.
*/

struct pc_s const *a_p = a;
struct pc_s const *b_p = b;

    if (a_p->count < b_p->count) {
	return 1;
    } else if (a_p->count > b_p->count) {
	return -1;
    } else {
	return 0;
    }
}


/******************************************************************************/
int main (int argc, char *argv[]) {

static struct FAB fab;

static VA_RANGE in_addr;
static VA_RANGE out_addr;

static struct sec_s sorted;

static struct sec_s *sec;

static int r0_status;
static int i;
static int count;

static const unsigned int prot = 0x00FF;

static $DESCRIPTOR (section_name_d, "PCDUMP_SECTION");

    /*
    ** We need a filename.
    */
    if (argc < 2) {
	(void)fprintf (stderr,
		       "Usage: $ MCR %s filename\n",
		       argv[0]);
	exit (EXIT_FAILURE);
    }

    /*
    ** Set up a file access block to access the input file.
    */
    fab = cc$rms_fab;
    fab.fab$b_fns = strlen (argv[1]);
    fab.fab$l_fna = argv[1];
    fab.fab$l_alq = SECTION_SIZE;
    fab.fab$l_fop = FAB$M_UFO;
    fab.fab$b_fac = FAB$M_GET;
    fab.fab$b_rtv = -1;

    /*
    ** Open the input file for read.  Note the UFO option so we can map a
    ** section to it.
    */
    r0_status = sys$open (&fab);
    if (r0_status == RMS$_FNF) {
	(void)fprintf (stderr,
		       "File %s not found!\n",
		       argv[1]);
	exit (EXIT_FAILURE);
    } else {
        errchk_stop (r0_status);
    }

    /*
    ** Map the file into memory.
    */
    r0_status = sys$crmpsc (&in_addr,
                            &out_addr,
                            0,
                            SEC$M_EXPREG,
                            &section_name_d,
                            0,
                            0,
                            fab.fab$l_stv,
                            SECTION_SIZE,
                            0,
                            prot,
                            0);
    errchk_stop (r0_status);

    /*
    ** Set up a C pointer.
    */
    sec = out_addr.va_range$ps_start_va;

    /*
    ** Copy non-zero count slots to our local storage.
    */
    count = 0;
    for (i = 0; i < SLOT_COUNT; i++) {
	if (sec->slot[i].count > 0) {
	    sorted.slot[count].count = sec->slot[i].count;
	    sorted.slot[count].pc = sec->slot[i].pc;
	    count++;
	}
    }

    /*
    ** Sort our local storage descending.
    */
    qsort (sorted.slot, count, sizeof (struct pc_s), compare);

    /*
    ** Print out the result.
    */
    for (i = 0; i < count; i++) {
	printf ("%08X : %-i\n",
		sorted.slot[i].pc,
		sorted.slot[i].count);
    }
}
